import os
from datetime import datetime, timedelta

import pandas as pd
import tushare as ts
import yaml
import shutil
from sqlalchemy import create_engine
from sqlalchemy import text

# 模块级单例
_config = None
_engine = None


def load_config():
    global _config
    if _config is not None:
        return _config

    config_path = 'config.yaml'
    template_path = 'config_template.yaml'

    if not os.path.exists(config_path):
        if os.path.exists(template_path):
            # 如果配置文件不存在但模板存在，则复制模板
            import shutil
            shutil.copy(template_path, config_path)
            print(f"已从 {template_path} 创建新的配置文件 {config_path}，请完善其中的配置信息后再运行。")
            exit(1)
        else:
            print(f"错误：配置文件 {config_path} 和模板 {template_path} 均不存在。")
            print(f"请创建 {template_path} 文件后再运行。")
            exit(1)

    with open(config_path, 'r') as f:
        _config = yaml.safe_load(f)
    return _config


def get_db_engine():
    """获取SQLite数据库引擎，如果不存在则创建"""
    global _engine
    if _engine is not None:
        return _engine

    config = load_config()
    db_path = config['sqlite']['path']  # 使用新的配置格式

    # 确保数据库目录存在
    os.makedirs(os.path.dirname(db_path), exist_ok=True)

    # 创建SQLite数据库引擎
    _engine = create_engine(f'sqlite:///{db_path}', echo=False)
    return _engine


def get_table_name(key):
    """根据表键名获取实际表名"""
    config = load_config()
    return config['sqlite']['table_name'].get(key, key)


def get_existing_trade_dates(table_key):
    """
    从数据库中获取已有的交易日期
    参数:
    table_key (str): 数据表键名
    返回:
    set: 已存在于数据库中的交易日期集合
    """
    table_name = get_table_name(table_key)
    engine = get_db_engine()
    query = f"SELECT DISTINCT trade_date FROM {table_name}"
    try:
        with engine.connect() as connection:
            result = connection.execute(text(query))
            return {row[0] for row in result}
    except Exception as e:
        print(f"获取已存在交易日期时出错: {e}")
        return set()


def load_df_from_db(table_key, conditions=None):
    """
    从数据库中加载数据
    参数:
    table_key (str): 表键名
    conditions (str): 过滤条件，如 "trade_date > '20230101'"
    返回:
    pandas.DataFrame: 查询结果
    """
    table_name = get_table_name(table_key)
    engine = get_db_engine()
    query = f"SELECT * FROM {table_name}"
    if conditions:
        query += f" WHERE {conditions}"
    return pd.read_sql(query, engine)


def save_df_to_db(df, table_key, if_exists='append'):
    """
    保存DataFrame到数据库
    参数:
    df (pandas.DataFrame): 要保存的数据
    table_key (str): 表键名
    if_exists (str): 如果表存在时的操作: 'fail', 'replace', 或 'append'
    """
    table_name = get_table_name(table_key)
    engine = get_db_engine()
    df.to_sql(table_name, engine, if_exists=if_exists, index=False)


def get_trade_cal(start_date=None, end_date=None):
    """
    获取指定时间段内的交易日历

    参数：
    start_date (str): 开始日期，格式'YYYYMMDD'
    end_date (str): 结束日期，格式'YYYYMMDD'

    返回：
    pandas.DataFrame: 交易日历
    """
    if start_date is None:
        start_date = (datetime.now() - timedelta(days=30)).strftime('%Y%m%d')
    if end_date is None:
        end_date = datetime.now().strftime('%Y%m%d')

    pro = ts.pro_api()
    trade_cal_df = pro.trade_cal(exchange='', start_date=start_date, end_date=end_date)
    return trade_cal_df[trade_cal_df['is_open'] == 1]['cal_date'].tolist()
